

import { Service, ServiceBroker, Context } from 'moleculer';
import { Container } from 'inversify';
import { OutputCommands, TYPES, NATS_HEMERA_KUBE_OUTPUT } from '../constants';
import { IOutputSettings, IOutputService, INewOutputPod } from '../interfaces';
import {
    IConfigurationService, TYPES as configTypes,
} from '@workbench-stack/core';

export class OuputMoleculerService extends Service {


    constructor(broker: ServiceBroker, { container, ...settings }: { container: Container }) {
        super(broker);
        const topic = NATS_HEMERA_KUBE_OUTPUT;

        const configuration = container.get<IConfigurationService>(configTypes.IConfigurationService);
        const outputService = container.get<IOutputService>(TYPES.IOutputService);

        this.parseServiceSchema({
            name: topic,
            actions: {
                [OutputCommands.ADD_OUTPUT_POD]: {
                    handler: async (ctx: Context<{ podName: string, newItem: INewOutputPod }>) => {
                        return await this.outputService.addNewType(ctx.params.podName, ctx.params.newItem);
                    },
                },
                [OutputCommands.GET_OUTPUT_PODS]: {
                    handler: async (ctx: Context<{}>) => {
                        return await this.outputService.getPods();
                    },
                },
                [OutputCommands.GET_OUTPUT_DATA]: {
                    handler: async (ctx: Context<{request: string}>) => {
                        return await this.outputService.getOutput(ctx.params.request);
                    },
                },
                [OutputCommands.UNWATCH_OUTPUT]: {
                    handler: async (ctx: Context<{}>) => {
                        return await this.outputService.unWatchLogs();
                    },
                },
                [OutputCommands.DISPOSE_OUTPUT]: {
                    handler: async (ctx: Context<{}>) => {
                        return await this.outputService.dispose();
                    },
                },
            },
        });
    }


}