import { injectable, inject } from 'inversify';
import * as Logger from 'bunyan';
import {forEach} from 'lodash';
import { IOutputService, IPodsRes, INewOutputPod } from '../interfaces';
const cmd = require('node-cmd');

export enum COMMAND  {
  STERN = 'stern',
}

export enum SternFlags {
  LABEL = '-l',
  TAIL = '--tail',
  STREAM = '-f',
  NAMESPACE = '-n',
  CONTAINER = '-c',
  TIMESTAMPS = '--timestamps',
}


@injectable()
export class OutputService implements IOutputService {

  private watcher: any;
  private timerId: any;
  private outputTypes: any = {};
  constructor(
    @inject('Logger') private logger: Logger,
    @inject('PubSub') private pubsub,
  ) {
  }

  public addNewType(podName, newItem: INewOutputPod): void {
    this.outputTypes[podName] = newItem;
  }

  public getPods(): IPodsRes[] {
    const podsObj = [];
    forEach(Object.keys(this.outputTypes), el => {
      const currentPod = this.outputTypes[el];
      podsObj.push({name: currentPod.name, title: currentPod.title ? currentPod.title : currentPod.name});
    });
    return podsObj;
  }

  public getOutput(pod: string) {
      this.watchLogs(pod);
      return {output: '', err: '', name: pod};
  }

  public unWatchLogs(): Boolean {
    if (this.watcher) {
      clearInterval(this.timerId);
      this.timerId = null;
      this.watcher.kill();
    }
    return true;
  }

  public dispose() {
     this.unWatchLogs();
  }

  private watchLogs(pod: string) {
    const request = this.outputTypes[pod];
    let args = [request.podQuery];
    if (request.container) {
      args.push(SternFlags.CONTAINER);
      args.push(request.container);
    }
    args = args.concat([SternFlags.NAMESPACE, request.nameSpace, '-t']);
    this.unWatchLogs();
    console.log('this.command', 'stern ' + args.join(' '));
    this.watcher = cmd.get('stern ' + args.join(' '));
    let res = '';

    this.watcher.stdout.on('data', (data) => {
      // console.log('data', data.toString());
      const dataStr = data.toString();
      if ((res.length + dataStr.length) * 2 > 1000000) {
        this.sendSubscribeData(res, request.name);
        res = '';
      }
      res = res + dataStr;

      if (!this.timerId) {
        this.sendSubscribeData(res, request.name);
        res = '';
        this.timerId = setInterval(() => {
          if (res.length) {
            this.sendSubscribeData(res, request.name);
            res = '';
          }
        }, 3000);
      }
    });

    this.watcher.stderr.on('data', (err) => {
        this.logger.debug(`stderr: %j`, err.toString());
        this.pubsub.publish(`OUTPUT_DATA_SUBSCRIPTIONS`, {
          outputDataSubscription: {
            output: '',
            err: err.toString(),
            name: request.name,
          },
        });
        //try run command again
      //  setTimeout(() => {
      //   console.log('request', request);
      //   this.watchLogs(request); }, 3000);
    });
  }

  private sendSubscribeData(data, name) {
      this.pubsub.publish(`OUTPUT_DATA_SUBSCRIPTIONS`, {
        outputDataSubscription: {
          output: data,
          err: '',
          name: name,
        },
      });
  }

}
